package javax.imageio;

import java.util.Arrays;
import javax.imageio.spi.ServiceRegistry;
import java.io.OutputStream;
import java.awt.image.RenderedImage;
import java.net.URL;
import java.io.InputStream;
import java.awt.image.BufferedImage;
import javax.imageio.spi.ImageTranscoderSpi;
import javax.imageio.spi.ImageWriterSpi;
import javax.imageio.spi.ImageReaderSpi;
import java.util.Collections;
import java.util.HashSet;
import javax.imageio.spi.ImageReaderWriterSpi;
import javax.imageio.spi.ImageOutputStreamSpi;
import javax.imageio.stream.ImageOutputStream;
import java.util.Iterator;
import java.io.IOException;
import javax.imageio.spi.ImageInputStreamSpi;
import javax.imageio.stream.ImageInputStream;
import java.io.FilePermission;
import java.io.File;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import sun.awt.AppContext;
import java.lang.reflect.Method;
import javax.imageio.spi.IIORegistry;

public final class ImageIO
{
    private static final IIORegistry theRegistry;
    private static Method readerFormatNamesMethod;
    private static Method readerFileSuffixesMethod;
    private static Method readerMIMETypesMethod;
    private static Method writerFormatNamesMethod;
    private static Method writerFileSuffixesMethod;
    private static Method writerMIMETypesMethod;
    
    private ImageIO() {
    }
    
    public static void scanForPlugins() {
        ImageIO.theRegistry.registerApplicationClasspathSpis();
    }
    
    private static synchronized ImageIO.CacheInfo getCacheInfo() {
        AppContext appContext = AppContext.getAppContext();
        ImageIO.CacheInfo value = (ImageIO.CacheInfo)appContext.get(ImageIO.CacheInfo.class);
        if (value == null) {
            value = new ImageIO.CacheInfo();
            appContext.put(ImageIO.CacheInfo.class, value);
            return value;
        }
        return value;
    }
    
    private static String getTempDir() {
        return (String)AccessController.doPrivileged(new GetPropertyAction("java.io.tmpdir"));
    }
    
    private static boolean hasCachePermission() {
        Boolean hasPermission = getCacheInfo().getHasPermission();
        if (hasPermission != null) {
            return (boolean)hasPermission;
        }
        try {
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                File cacheDirectory = getCacheDirectory();
                String s;
                if (cacheDirectory != null) {
                    s = cacheDirectory.getPath();
                }
                else {
                    s = getTempDir();
                    if (s == null || s.isEmpty()) {
                        getCacheInfo().setHasPermission(Boolean.FALSE);
                        return false;
                    }
                }
                if (!s.endsWith(File.separator)) {
                    s += File.separator;
                }
                securityManager.checkPermission(new FilePermission(s + "*", "read, write, delete"));
            }
        }
        catch (SecurityException ex) {
            getCacheInfo().setHasPermission(Boolean.FALSE);
            return false;
        }
        getCacheInfo().setHasPermission(Boolean.TRUE);
        return true;
    }
    
    public static void setUseCache(boolean useCache) {
        getCacheInfo().setUseCache(useCache);
    }
    
    public static boolean getUseCache() {
        return getCacheInfo().getUseCache();
    }
    
    public static void setCacheDirectory(File cacheDirectory) {
        if (cacheDirectory != null && !cacheDirectory.isDirectory()) {
            throw new IllegalArgumentException("Not a directory!");
        }
        getCacheInfo().setCacheDirectory(cacheDirectory);
        getCacheInfo().setHasPermission(null);
        return;
    }
    
    public static File getCacheDirectory() {
        return getCacheInfo().getCacheDirectory();
    }
    
    public static ImageInputStream createImageInputStream(Object o) throws IOException {
        if (o == null) {
            throw new IllegalArgumentException("input == null!");
        }
        Iterator<ImageInputStreamSpi> serviceProviders;
        try {
            serviceProviders = ImageIO.theRegistry.getServiceProviders(ImageInputStreamSpi.class, true);
        }
        catch (IllegalArgumentException ex) {
            return null;
        }
        int n = getUseCache() && hasCachePermission();
        while (serviceProviders.hasNext()) {
            ImageInputStreamSpi imageInputStreamSpi = (ImageInputStreamSpi)serviceProviders.next();
            if (imageInputStreamSpi.getInputClass().isInstance(o)) {
                try {
                    return imageInputStreamSpi.createInputStreamInstance(o, (boolean)n, getCacheDirectory());
                }
                catch (IOException ex2) {
                    throw new IIOException("Can't create cache file!", ex2);
                }
            }
            else {
                continue;
            }
        }
        return null;
    }
    
    public static ImageOutputStream createImageOutputStream(Object o) throws IOException {
        if (o == null) {
            throw new IllegalArgumentException("output == null!");
        }
        Iterator<ImageOutputStreamSpi> serviceProviders;
        try {
            serviceProviders = ImageIO.theRegistry.getServiceProviders(ImageOutputStreamSpi.class, true);
        }
        catch (IllegalArgumentException ex) {
            return null;
        }
        int n = getUseCache() && hasCachePermission();
        while (serviceProviders.hasNext()) {
            ImageOutputStreamSpi imageOutputStreamSpi = (ImageOutputStreamSpi)serviceProviders.next();
            if (imageOutputStreamSpi.getOutputClass().isInstance(o)) {
                try {
                    return imageOutputStreamSpi.createOutputStreamInstance(o, (boolean)n, getCacheDirectory());
                }
                catch (IOException ex2) {
                    throw new IIOException("Can't create cache file!", ex2);
                }
            }
            else {
                continue;
            }
        }
        return null;
    }
    
    private static <S extends ImageReaderWriterSpi> String[] getReaderWriterInfo(Class<S> category, ImageIO.SpiInfo spiInfo) {
        Iterator<S> serviceProviders;
        try {
            serviceProviders = ImageIO.theRegistry.getServiceProviders(category, true);
        }
        catch (IllegalArgumentException ex) {
            return new String[0];
        }
        HashSet<Object> c = new HashSet();
        while (serviceProviders.hasNext()) {
            Collections.addAll(c, spiInfo.info((S)serviceProviders.next()));
        }
        return (String[])c.toArray(new String[c.size()]);
    }
    
    public static String[] getReaderFormatNames() {
        return getReaderWriterInfo(ImageReaderSpi.class, ImageIO.SpiInfo.FORMAT_NAMES);
    }
    
    public static String[] getReaderMIMETypes() {
        return getReaderWriterInfo(ImageReaderSpi.class, ImageIO.SpiInfo.MIME_TYPES);
    }
    
    public static String[] getReaderFileSuffixes() {
        return getReaderWriterInfo(ImageReaderSpi.class, ImageIO.SpiInfo.FILE_SUFFIXES);
    }
    
    public static Iterator<ImageReader> getImageReaders(Object o) {
        if (o == null) {
            throw new IllegalArgumentException("input == null!");
        }
        Iterator<ImageReaderSpi> serviceProviders;
        try {
            serviceProviders = ImageIO.theRegistry.getServiceProviders(ImageReaderSpi.class, new CanDecodeInputFilter(o), true);
        }
        catch (IllegalArgumentException ex) {
            return Collections.emptyIterator();
        }
        return new ImageReaderIterator(serviceProviders);
    }
    
    public static Iterator<ImageReader> getImageReadersByFormatName(String s) {
        if (s == null) {
            throw new IllegalArgumentException("formatName == null!");
        }
        Iterator<ImageReaderSpi> serviceProviders;
        try {
            serviceProviders = ImageIO.theRegistry.getServiceProviders(ImageReaderSpi.class, new ImageIO.ContainsFilter(ImageIO.readerFormatNamesMethod, s), true);
        }
        catch (IllegalArgumentException ex) {
            return Collections.emptyIterator();
        }
        return new ImageReaderIterator(serviceProviders);
    }
    
    public static Iterator<ImageReader> getImageReadersBySuffix(String s) {
        if (s == null) {
            throw new IllegalArgumentException("fileSuffix == null!");
        }
        Iterator<ImageReaderSpi> serviceProviders;
        try {
            serviceProviders = ImageIO.theRegistry.getServiceProviders(ImageReaderSpi.class, new ImageIO.ContainsFilter(ImageIO.readerFileSuffixesMethod, s), true);
        }
        catch (IllegalArgumentException ex) {
            return Collections.emptyIterator();
        }
        return new ImageReaderIterator(serviceProviders);
    }
    
    public static Iterator<ImageReader> getImageReadersByMIMEType(String s) {
        if (s == null) {
            throw new IllegalArgumentException("MIMEType == null!");
        }
        Iterator<ImageReaderSpi> serviceProviders;
        try {
            serviceProviders = ImageIO.theRegistry.getServiceProviders(ImageReaderSpi.class, new ImageIO.ContainsFilter(ImageIO.readerMIMETypesMethod, s), true);
        }
        catch (IllegalArgumentException ex) {
            return Collections.emptyIterator();
        }
        return new ImageReaderIterator(serviceProviders);
    }
    
    public static String[] getWriterFormatNames() {
        return getReaderWriterInfo(ImageWriterSpi.class, ImageIO.SpiInfo.FORMAT_NAMES);
    }
    
    public static String[] getWriterMIMETypes() {
        return getReaderWriterInfo(ImageWriterSpi.class, ImageIO.SpiInfo.MIME_TYPES);
    }
    
    public static String[] getWriterFileSuffixes() {
        return getReaderWriterInfo(ImageWriterSpi.class, ImageIO.SpiInfo.FILE_SUFFIXES);
    }
    
    private static boolean contains(String[] array, String s) {
        int i = 0;
        while (i < array.length) {
            if (s.equalsIgnoreCase(array[i])) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    public static Iterator<ImageWriter> getImageWritersByFormatName(String s) {
        if (s == null) {
            throw new IllegalArgumentException("formatName == null!");
        }
        Iterator<ImageWriterSpi> serviceProviders;
        try {
            serviceProviders = ImageIO.theRegistry.getServiceProviders(ImageWriterSpi.class, new ImageIO.ContainsFilter(ImageIO.writerFormatNamesMethod, s), true);
        }
        catch (IllegalArgumentException ex) {
            return Collections.emptyIterator();
        }
        return new ImageWriterIterator(serviceProviders);
    }
    
    public static Iterator<ImageWriter> getImageWritersBySuffix(String s) {
        if (s == null) {
            throw new IllegalArgumentException("fileSuffix == null!");
        }
        Iterator<ImageWriterSpi> serviceProviders;
        try {
            serviceProviders = ImageIO.theRegistry.getServiceProviders(ImageWriterSpi.class, new ImageIO.ContainsFilter(ImageIO.writerFileSuffixesMethod, s), true);
        }
        catch (IllegalArgumentException ex) {
            return Collections.emptyIterator();
        }
        return new ImageWriterIterator(serviceProviders);
    }
    
    public static Iterator<ImageWriter> getImageWritersByMIMEType(String s) {
        if (s == null) {
            throw new IllegalArgumentException("MIMEType == null!");
        }
        Iterator<ImageWriterSpi> serviceProviders;
        try {
            serviceProviders = ImageIO.theRegistry.getServiceProviders(ImageWriterSpi.class, new ImageIO.ContainsFilter(ImageIO.writerMIMETypesMethod, s), true);
        }
        catch (IllegalArgumentException ex) {
            return Collections.emptyIterator();
        }
        return new ImageWriterIterator(serviceProviders);
    }
    
    public static ImageWriter getImageWriter(ImageReader reader) {
        if (reader == null) {
            throw new IllegalArgumentException("reader == null!");
        }
        ImageReaderSpi originatingProvider = reader.getOriginatingProvider();
        if (originatingProvider == null) {
            Iterator<ImageReaderSpi> serviceProviders;
            try {
                serviceProviders = ImageIO.theRegistry.getServiceProviders(ImageReaderSpi.class, false);
            }
            catch (IllegalArgumentException ex) {
                return null;
            }
            while (serviceProviders.hasNext()) {
                ImageReaderSpi imageReaderSpi = (ImageReaderSpi)serviceProviders.next();
                if (imageReaderSpi.isOwnReader(reader)) {
                    originatingProvider = imageReaderSpi;
                    break;
                }
                continue;
            }
            if (originatingProvider == null) {
                return null;
            }
        }
        String[] imageWriterSpiNames = originatingProvider.getImageWriterSpiNames();
        if (imageWriterSpiNames == null) {
            return null;
        }
        Class<ImageWriterSpi> forName;
        try {
            forName = Class.forName(imageWriterSpiNames[0], true, ClassLoader.getSystemClassLoader());
        }
        catch (ClassNotFoundException ex2) {
            return null;
        }
        ImageWriterSpi provider = (ImageWriterSpi)ImageIO.theRegistry.getServiceProviderByClass(forName);
        if (provider == null) {
            return null;
        }
        try {
            return provider.createWriterInstance();
        }
        catch (IOException ex3) {
            ImageIO.theRegistry.deregisterServiceProvider(provider, ImageWriterSpi.class);
            return null;
        }
    }
    
    public static ImageReader getImageReader(ImageWriter writer) {
        if (writer == null) {
            throw new IllegalArgumentException("writer == null!");
        }
        ImageWriterSpi originatingProvider = writer.getOriginatingProvider();
        if (originatingProvider == null) {
            Iterator<ImageWriterSpi> serviceProviders;
            try {
                serviceProviders = ImageIO.theRegistry.getServiceProviders(ImageWriterSpi.class, false);
            }
            catch (IllegalArgumentException ex) {
                return null;
            }
            while (serviceProviders.hasNext()) {
                ImageWriterSpi imageWriterSpi = (ImageWriterSpi)serviceProviders.next();
                if (imageWriterSpi.isOwnWriter(writer)) {
                    originatingProvider = imageWriterSpi;
                    break;
                }
                continue;
            }
            if (originatingProvider == null) {
                return null;
            }
        }
        String[] imageReaderSpiNames = originatingProvider.getImageReaderSpiNames();
        if (imageReaderSpiNames == null) {
            return null;
        }
        Class<ImageReaderSpi> forName;
        try {
            forName = Class.forName(imageReaderSpiNames[0], true, ClassLoader.getSystemClassLoader());
        }
        catch (ClassNotFoundException ex2) {
            return null;
        }
        ImageReaderSpi provider = (ImageReaderSpi)ImageIO.theRegistry.getServiceProviderByClass(forName);
        if (provider == null) {
            return null;
        }
        try {
            return provider.createReaderInstance();
        }
        catch (IOException ex3) {
            ImageIO.theRegistry.deregisterServiceProvider(provider, ImageReaderSpi.class);
            return null;
        }
    }
    
    public static Iterator<ImageWriter> getImageWriters(ImageTypeSpecifier imageTypeSpecifier, String s) {
        if (imageTypeSpecifier == null) {
            throw new IllegalArgumentException("type == null!");
        }
        if (s == null) {
            throw new IllegalArgumentException("formatName == null!");
        }
        Iterator<ImageWriterSpi> serviceProviders;
        try {
            serviceProviders = ImageIO.theRegistry.getServiceProviders(ImageWriterSpi.class, new ImageIO.CanEncodeImageAndFormatFilter(imageTypeSpecifier, s), true);
        }
        catch (IllegalArgumentException ex) {
            return Collections.emptyIterator();
        }
        return new ImageWriterIterator(serviceProviders);
    }
    
    public static Iterator<ImageTranscoder> getImageTranscoders(ImageReader imageReader, ImageWriter imageWriter) {
        if (imageReader == null) {
            throw new IllegalArgumentException("reader == null!");
        }
        if (imageWriter == null) {
            throw new IllegalArgumentException("writer == null!");
        }
        ImageIO.TranscoderFilter filter = new ImageIO.TranscoderFilter(imageReader.getOriginatingProvider(), imageWriter.getOriginatingProvider());
        Iterator<ImageTranscoderSpi> serviceProviders;
        try {
            serviceProviders = ImageIO.theRegistry.getServiceProviders(ImageTranscoderSpi.class, filter, true);
        }
        catch (IllegalArgumentException ex) {
            return Collections.emptyIterator();
        }
        return new ImageTranscoderIterator(serviceProviders);
    }
    
    public static BufferedImage read(File file) throws IOException {
        if (file == null) {
            throw new IllegalArgumentException("input == null!");
        }
        if (!file.canRead()) {
            throw new IIOException("Can't read input file!");
        }
        ImageInputStream imageInputStream = createImageInputStream(file);
        if (imageInputStream == null) {
            throw new IIOException("Can't create an ImageInputStream!");
        }
        BufferedImage read = read(imageInputStream);
        if (read == null) {
            imageInputStream.close();
            return read;
        }
        return read;
    }
    
    public static BufferedImage read(InputStream inputStream) throws IOException {
        if (inputStream == null) {
            throw new IllegalArgumentException("input == null!");
        }
        ImageInputStream imageInputStream = createImageInputStream(inputStream);
        BufferedImage read = read(imageInputStream);
        if (read == null) {
            imageInputStream.close();
            return read;
        }
        return read;
    }
    
    public static BufferedImage read(URL url) throws IOException {
        if (url == null) {
            throw new IllegalArgumentException("input == null!");
        }
        InputStream openStream;
        try {
            openStream = url.openStream();
        }
        catch (IOException ex) {
            throw new IIOException("Can't get input stream from URL!", ex);
        }
        ImageInputStream imageInputStream = createImageInputStream(openStream);
        BufferedImage read;
        try {
            read = read(imageInputStream);
            if (read == null) {
                imageInputStream.close();
            }
        }
        finally {
            openStream.close();
        }
        return read;
    }
    
    public static BufferedImage read(ImageInputStream imageInputStream) throws IOException {
        if (imageInputStream == null) {
            throw new IllegalArgumentException("stream == null!");
        }
        Iterator<ImageReader> imageReaders = getImageReaders(imageInputStream);
        if (!imageReaders.hasNext()) {
            return null;
        }
        ImageReader imageReader = (ImageReader)imageReaders.next();
        ImageReadParam defaultReadParam = imageReader.getDefaultReadParam();
        imageReader.setInput(imageInputStream, true, true);
        BufferedImage read;
        try {
            read = imageReader.read(0, defaultReadParam);
        }
        finally {
            imageReader.dispose();
            imageInputStream.close();
        }
        return read;
    }
    
    public static boolean write(RenderedImage renderedImage, String s, ImageOutputStream imageOutputStream) throws IOException {
        if (renderedImage == null) {
            throw new IllegalArgumentException("im == null!");
        }
        if (s == null) {
            throw new IllegalArgumentException("formatName == null!");
        }
        if (imageOutputStream == null) {
            throw new IllegalArgumentException("output == null!");
        }
        return doWrite(renderedImage, getWriter(renderedImage, s), imageOutputStream);
    }
    
    public static boolean write(RenderedImage renderedImage, String s, File file) throws IOException {
        if (file == null) {
            throw new IllegalArgumentException("output == null!");
        }
        ImageWriter writer = getWriter(renderedImage, s);
        if (writer == null) {
            return false;
        }
        ImageOutputStream imageOutputStream;
        try {
            file.delete();
            imageOutputStream = createImageOutputStream(file);
        }
        catch (IOException ex) {
            throw new IIOException("Can't create output stream!", ex);
        }
        try {
            return doWrite(renderedImage, writer, imageOutputStream);
        }
        finally {
            imageOutputStream.close();
        }
    }
    
    public static boolean write(RenderedImage renderedImage, String s, OutputStream outputStream) throws IOException {
        if (outputStream == null) {
            throw new IllegalArgumentException("output == null!");
        }
        ImageOutputStream imageOutputStream;
        try {
            imageOutputStream = createImageOutputStream(outputStream);
        }
        catch (IOException ex) {
            throw new IIOException("Can't create output stream!", ex);
        }
        try {
            return doWrite(renderedImage, getWriter(renderedImage, s), imageOutputStream);
        }
        finally {
            imageOutputStream.close();
        }
    }
    
    private static ImageWriter getWriter(RenderedImage renderedImage, String s) {
        Iterator<ImageWriter> imageWriters = getImageWriters(ImageTypeSpecifier.createFromRenderedImage(renderedImage), s);
        if (imageWriters.hasNext()) {
            return (ImageWriter)imageWriters.next();
        }
        return null;
    }
    
    private static boolean doWrite(RenderedImage renderedImage, ImageWriter imageWriter, ImageOutputStream output) throws IOException {
        if (imageWriter == null) {
            return false;
        }
        imageWriter.setOutput(output);
        try {
            imageWriter.write(renderedImage);
        }
        finally {
            imageWriter.dispose();
            output.flush();
        }
        return true;
    }
    
    static boolean access$200(String[] array, String s) {
        return contains(array, s);
    }
    
    static {
        theRegistry = IIORegistry.getDefaultInstance();
        try {
            ImageIO.readerFormatNamesMethod = ImageReaderSpi.class.getMethod("getFormatNames", new Class[0]);
            ImageIO.readerFileSuffixesMethod = ImageReaderSpi.class.getMethod("getFileSuffixes", new Class[0]);
            ImageIO.readerMIMETypesMethod = ImageReaderSpi.class.getMethod("getMIMETypes", new Class[0]);
            ImageIO.writerFormatNamesMethod = ImageWriterSpi.class.getMethod("getFormatNames", new Class[0]);
            ImageIO.writerFileSuffixesMethod = ImageWriterSpi.class.getMethod("getFileSuffixes", new Class[0]);
            ImageIO.writerMIMETypesMethod = ImageWriterSpi.class.getMethod("getMIMETypes", new Class[0]);
        }
        catch (NoSuchMethodException ex) {
            ex.printStackTrace();
        }
    }
    
    static class CacheInfo
    {
        boolean useCache;
        File cacheDirectory;
        Boolean hasPermission;
        
        public CacheInfo() {
            this.useCache = true;
            this.cacheDirectory = null;
            this.hasPermission = null;
        }
        
        public boolean getUseCache() {
            return this.useCache;
        }
        
        public void setUseCache(boolean useCache) {
            this.useCache = useCache;
        }
        
        public File getCacheDirectory() {
            return this.cacheDirectory;
        }
        
        public void setCacheDirectory(File cacheDirectory) {
            this.cacheDirectory = cacheDirectory;
        }
        
        public Boolean getHasPermission() {
            return this.hasPermission;
        }
        
        public void setHasPermission(Boolean hasPermission) {
            this.hasPermission = hasPermission;
        }
    }
    
    private enum SpiInfo
    {
        FORMAT_NAMES {
            @Override
            String[] info(ImageReaderWriterSpi imageReaderWriterSpi) {
                return imageReaderWriterSpi.getFormatNames();
            }
        }, 
        MIME_TYPES {
            @Override
            String[] info(ImageReaderWriterSpi imageReaderWriterSpi) {
                return imageReaderWriterSpi.getMIMETypes();
            }
        }, 
        FILE_SUFFIXES {
            @Override
            String[] info(ImageReaderWriterSpi imageReaderWriterSpi) {
                return imageReaderWriterSpi.getFileSuffixes();
            }
        };
        
        abstract String[] info(ImageReaderWriterSpi p0);
    }
    
    static class ImageReaderIterator implements Iterator<ImageReader>
    {
        public Iterator iter;
        
        public ImageReaderIterator(Iterator iter) {
            this.iter = iter;
        }
        
        @Override
        public boolean hasNext() {
            return this.iter.hasNext();
        }
        
        @Override
        public ImageReader next() {
            ImageReaderSpi provider = null;
            try {
                provider = (ImageReaderSpi)this.iter.next();
                return provider.createReaderInstance();
            }
            catch (IOException ex) {
                ImageIO.theRegistry.deregisterServiceProvider(provider, ImageReaderSpi.class);
                return null;
            }
        }
        
        @Override
        public void remove() {
            throw new UnsupportedOperationException();
        }
    }
    
    static class CanDecodeInputFilter implements ServiceRegistry.Filter
    {
        Object input;
        
        public CanDecodeInputFilter(Object input) {
            this.input = input;
        }
        
        @Override
        public boolean filter(Object o) {
            try {
                ImageReaderSpi imageReaderSpi = (ImageReaderSpi)o;
                ImageInputStream imageInputStream = null;
                if ((this.input instanceof ImageInputStream)) {
                    imageInputStream = (ImageInputStream)this.input;
                }
                if (imageInputStream != null) {
                    imageInputStream.mark();
                }
                int canDecodeInput = imageReaderSpi.canDecodeInput(this.input);
                if (imageInputStream != null) {
                    imageInputStream.reset();
                }
                return canDecodeInput;
            }
            catch (IOException ex) {
                return false;
            }
        }
    }
    
    static class CanEncodeImageAndFormatFilter implements ServiceRegistry.Filter
    {
        ImageTypeSpecifier type;
        String formatName;
        
        public CanEncodeImageAndFormatFilter(ImageTypeSpecifier type, String formatName) {
            this.type = type;
            this.formatName = formatName;
        }
        
        @Override
        public boolean filter(Object o) {
            ImageWriterSpi imageWriterSpi = (ImageWriterSpi)o;
            return Arrays.asList(imageWriterSpi.getFormatNames()).contains(this.formatName) && imageWriterSpi.canEncodeImage(this.type);
        }
    }
    
    static class ContainsFilter implements ServiceRegistry.Filter
    {
        Method method;
        String name;
        
        public ContainsFilter(Method method, String name) {
            this.method = method;
            this.name = name;
        }
        
        @Override
        public boolean filter(Object obj) {
            try {
                return ImageIO.access$200((String[])(String[])this.method.invoke(obj, new Object[0]), this.name);
            }
            catch (Exception ex) {
                return false;
            }
        }
    }
    
    static class ImageWriterIterator implements Iterator<ImageWriter>
    {
        public Iterator iter;
        
        public ImageWriterIterator(Iterator iter) {
            this.iter = iter;
        }
        
        @Override
        public boolean hasNext() {
            return this.iter.hasNext();
        }
        
        @Override
        public ImageWriter next() {
            ImageWriterSpi provider = null;
            try {
                provider = (ImageWriterSpi)this.iter.next();
                return provider.createWriterInstance();
            }
            catch (IOException ex) {
                ImageIO.theRegistry.deregisterServiceProvider(provider, ImageWriterSpi.class);
                return null;
            }
        }
        
        @Override
        public void remove() {
            throw new UnsupportedOperationException();
        }
    }
    
    static class ImageTranscoderIterator implements Iterator<ImageTranscoder>
    {
        public Iterator iter;
        
        public ImageTranscoderIterator(Iterator iter) {
            this.iter = iter;
        }
        
        @Override
        public boolean hasNext() {
            return this.iter.hasNext();
        }
        
        @Override
        public ImageTranscoder next() {
            return ((ImageTranscoderSpi)this.iter.next()).createTranscoderInstance();
        }
        
        @Override
        public void remove() {
            throw new UnsupportedOperationException();
        }
    }
    
    static class TranscoderFilter implements ServiceRegistry.Filter
    {
        String readerSpiName;
        String writerSpiName;
        
        public TranscoderFilter(ImageReaderSpi imageReaderSpi, ImageWriterSpi imageWriterSpi) {
            this.readerSpiName = imageReaderSpi.getClass().getName();
            this.writerSpiName = imageWriterSpi.getClass().getName();
        }
        
        @Override
        public boolean filter(Object o) {
            ImageTranscoderSpi imageTranscoderSpi = (ImageTranscoderSpi)o;
            String readerServiceProviderName = imageTranscoderSpi.getReaderServiceProviderName();
            String writerServiceProviderName = imageTranscoderSpi.getWriterServiceProviderName();
            return readerServiceProviderName.equals(this.readerSpiName) && writerServiceProviderName.equals(this.writerSpiName);
        }
    }
}
